/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS          All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using exo_jcr.webdav.csclient.Response;
using exo_jcr.webdav.csclient.Request;
using exo_jcr.webdav.csclient.Commands;
using exo_jcr.webdav.csclient.DavProperties;
using exo_jcr.webdav.csclient;

/**
 * Created by The eXo Platform SARL
 * Authors : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 *         : Max Shaposhnik <uy7c@yahoo.com>
 * @version $Id:
 */

namespace exo_jcr.msofficeplugin
{
    public partial class ActiveTab : UserControl, ActiveAppContext
    {
        private AppWindowContext appContext;

        public ActiveTab()
        {
            InitializeComponent();
        }

        public void setAppContext(AppWindowContext appContext)
        {
            this.appContext = appContext;
            eXoFileListControl1.setAppContext(this);
            versionListControl1.setAppContext(this);
            propertyUpdateControl.setAppContext(this);
            wordControl1.setAppContext(this);
        }

        public AppWindowContext getWindowContext()
        {
            return appContext;
        }

        
        public ArrayList getWordDocs()
        {
            return appContext.getWebDavAppContext().getWordFiles();
        }

        public String getCurrentPath()
        {
            return comboPath.Text;
        }

        public void setCurrentPath(String path) {

            if (path != null && path != "")
            {
                comboPath.Text = path;
                this.lastReportedResource = path;
            }
        }
        
        private String lastReportedResource = "/";

        public void doRefreshVersionList()
        {
            doVersionList();
        }

        public void doRefreshPropertyList()
        {
            doPropertyList(lastReportedResource);
        }

        #region public void doVersionList(String path)
        public void doVersionList()
        {
            String path = getCurrentPath();

            this.lastReportedResource = path;

            tabControl1.SelectTab(1);
            versionListControl1.clearItems();

            DavContext context = appContext.getContext();
            try
            {
                ReportCommand cmdReport = new ReportCommand(context);
                cmdReport.setResourcePath(path);

                cmdReport.addRequiredProperty(DavProperty.DISPLAYNAME);
                cmdReport.addRequiredProperty(DavProperty.CREATIONDATE);
                cmdReport.addRequiredProperty(DavProperty.GETCONTENTLENGTH);

                int status = cmdReport.execute();

                if (status != DavStatus.MULTISTATUS) {
                    MessageBox.Show("STATUS: " + status.ToString());
                    return;
                }

                if (appContext.getVersionListTable()[path] != null) {
                    appContext.getVersionListTable().Remove(path);
                }

                appContext.getVersionListTable().Add(path, cmdReport.getMultistatus());
                versionListControl1.fillVersionList(cmdReport.getMultistatus());
            } catch (Exception exc) {
                MessageBox.Show("Cannot load paramethers,\n please run Settings first.", "Error",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        #endregion

        #region public void doPropertyList(String path)
        public void doPropertyList(String path) {
            tabControl1.SelectTab(3);
            propertyUpdateControl.clearItems();

            DavContext context = appContext.getContext();
            try
            {
                PropFindCommand propFind = new PropFindCommand(context);
                propFind.setResourcePath(path);

                propFind.addRequiredProperty(DavProperty.CHECKEDIN);
                propFind.addRequiredProperty(DavProperty.CHECKEDOUT);
                propFind.addRequiredProperty(DavProperty.CHILDCOUNT);
                propFind.addRequiredProperty(DavProperty.GETCONTENTLENGTH);
                propFind.addRequiredProperty(DavProperty.GETCONTENTTYPE);
                propFind.addRequiredProperty(DavProperty.CREATIONDATE);
                propFind.addRequiredProperty(DavProperty.HASCHILDREN);
                propFind.addRequiredProperty(DavProperty.DISPLAYNAME);
                propFind.addRequiredProperty(DavProperty.ISCOLLECTION);
                propFind.addRequiredProperty(DavProperty.ISFOLDER);
                propFind.addRequiredProperty(DavProperty.ISROOT);
                propFind.addRequiredProperty(DavProperty.ISVERSIONED);
                propFind.addRequiredProperty(DavProperty.GETLASTMODIFIED);
                propFind.addRequiredProperty(DavProperty.RESOURCETYPE);
                propFind.addRequiredProperty(DavProperty.SUPPORTEDLOCK);
                propFind.addRequiredProperty(DavProperty.SUPPORTEDQUERYGRAMMARSET);
                propFind.addRequiredProperty(DavProperty.SUPPORTEDMETHODSET);
                propFind.addRequiredProperty(DavProperty.VERSIONHISTORY);
                propFind.addRequiredProperty(DavProperty.VERSIONNAME);

                propFind.addRequiredProperty("dc:title");
                propFind.addRequiredProperty("dc:creator");
                propFind.addRequiredProperty("dc:subject");
                propFind.addRequiredProperty("dc:description");
                propFind.addRequiredProperty("dc:publisher");
                propFind.addRequiredProperty("dc:contributor");
                propFind.addRequiredProperty("dc:date");
                propFind.addRequiredProperty("dc:resourceType");
                propFind.addRequiredProperty("dc:format");
                propFind.addRequiredProperty("dc:identifier");
                propFind.addRequiredProperty("dc:source");
                propFind.addRequiredProperty("dc:language");
                propFind.addRequiredProperty("dc:relation");
                propFind.addRequiredProperty("dc:coverage");
                propFind.addRequiredProperty("dc:rights");


                int status = propFind.execute();
                if (status != DavStatus.MULTISTATUS)
                {
                    MessageBox.Show("STATUS: " + status.ToString());
                    return;
                }

                if (appContext.getVersionListTable()[path] != null)
                {
                    appContext.getVersionListTable().Remove(path);
                }

                appContext.getVersionListTable().Add(path, propFind.getMultistatus());
                propertyUpdateControl.fillPropsList(propFind.getMultistatus());

                //versionListControl1.fillVersionList(propFind.getMultistatus());
            }
            catch (Exception exc)
            {
                MessageBox.Show("Cannot load paramethers,\n please run Settings first.", "Error",
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        #endregion

        private Multistatus fileListControlCurrent = null;

        public void refreshFileList(String path)
        {
            Multistatus multistatus = (Multistatus)appContext.getMultistatusCache()[path];
            if (multistatus != null)
            {
                eXoFileListControl1.fillFileList(path, multistatus);
                fileListControlCurrent = multistatus;
            }
        }

        public void refreshFileList()
        {            
            refreshFileList(comboPath.Text);
            this.lastReportedResource = comboPath.Text;
        }

        public void doGetFile(String fileName, String href, String savePath)
        {
            //MessageBox.Show(" doGetFile - FILENAME: [" + fileName + "]");
            //MessageBox.Show(" doGetFile - HREF: [" + href + "]");
            //MessageBox.Show(" doGetFile - SAVEPATH: [" + savePath + "]");

            DavContext context = appContext.getContext();
            try
            {
                GetCommand get = new GetCommand(context);
                get.setResourcePath(href);

                int status = get.execute();
                if (status == DavStatus.OK)
                {
                    byte[] resp = get.getResponseBody();

                    if (File.Exists(fileName)) {
                        File.Delete(fileName);
                    }

                    FileStream fs = new FileStream(fileName, FileMode.CreateNew, FileAccess.Write, FileShare.Read);
                    BinaryWriter w = new BinaryWriter(fs);
                    for (long i = 0; i < resp.Length; i++)
                    {
                        w.Write(resp[i]);
                    }
                    w.Close();
                    fs.Close();
                }

            }
            catch (Exception ed)
            {
                MessageBox.Show("AT doGetFile " + ed.Message + ed.StackTrace);
                return;
            }

            WordFileInfo fileInfo = new WordFileInfo();
            fileInfo.FileName = fileName;
            fileInfo.RemoteFileName = savePath;
            fileInfo.FileAction = WordFileInfo.ACTION_OPEN;

            appContext.getWebDavAppContext().openWordDocument(fileInfo);
        }

        public void doGetFile(String remoteHref)
        {
            try
            {
                String href = remoteHref;

                href = href.Substring(appContext.getContext().getContextHref().Length);

                href = href.Replace("%3f", "?");
                href = href.Replace("%3d", "=");

                Environment.SpecialFolder p = Environment.SpecialFolder.Personal;
                String s_p = Environment.GetFolderPath(p);

                String f_name = href.Substring(href.LastIndexOf("/") + 1);
                f_name = f_name.Substring(0, f_name.IndexOf("?"));

                String FILE_NAME = s_p + "\\" + f_name;
                String repoPath = href.Substring(0, href.IndexOf("?"));

                doGetFile(FILE_NAME, href, repoPath);
            }
            catch (Exception exc)
            {
                MessageBox.Show("Can't open file: [" + remoteHref + "]");
            }
        }

        #region public void doGetFile(int recId)
        public void doGetFile(int recId)
        {
            if (fileListControlCurrent == null) {
                return;
            }

            ArrayList responses = fileListControlCurrent.getResponses();
            DavResponse response = (DavResponse)responses[recId];
            String href = response.getHref().getHref();

            Environment.SpecialFolder p = Environment.SpecialFolder.Personal;
            String s_p = Environment.GetFolderPath(p);
            String f_name = href.Substring(href.LastIndexOf("/") + 1);
            String FILE_NAME = s_p + "\\" + f_name;

            doGetFile(FILE_NAME, this.lastReportedResource, this.lastReportedResource);
        }
        #endregion

        #region public void doPutFile(String fileSystemName, String remoteFileName)
        public void doPutFile(String fileSystemName, String remoteFileName)
        {
            FileStream stream = new FileStream(fileSystemName, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
            long len = stream.Length;
            byte[] filedata = new byte[len];
            int readed = 0;
            while (readed < len) {
                readed += stream.Read(filedata, 0, (int)(len - readed));
            }

            DavContext context = appContext.getContext();
            PutCommand put = new PutCommand(context);

            put.setResourcePath(remoteFileName);
            put.setRequestBody(filedata);
            int status = put.execute();
            if (status != DavStatus.CREATED) {
                MessageBox.Show("CAN'T PUT FILE");
            }
        }
        #endregion

    }
}
