/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS          All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using exo_jcr.webdav.csclient.Response;
using exo_jcr.webdav.csclient.DavProperties;
using exo_jcr.webdav.csclient;

/**
 * Created by The eXo Platform SARL
 * Authors : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 *         : Max Shaposhnik <uy7c@yahoo.com>
 * @version $Id:
 */

namespace exo_jcr.msofficeplugin
{
    public partial class eXoFileListControl : UserControl
    {
 
        private ActiveAppContext activeAppContext;
    
        public eXoFileListControl()
        {
            InitializeComponent();
        }

        public void setAppContext(ActiveAppContext activeAppContext)
        {
            this.activeAppContext = activeAppContext;
        }

        private String remotePath = "";

        #region fillFileList(Multistatus multistatus)
        public void fillFileList(String remotePath, Multistatus multistatus)
        {
            this.remotePath = remotePath;
            listFiles.Items.Clear();

            try {
                ArrayList responses = multistatus.getResponses();
                for (int i = 0; i < responses.Count; i++)
                {
                    DavResponse response = (DavResponse)responses[i];

                    String displayName = "";
                    String created = "";
                    String modified = "";
                    String size = "";

                    DisplayNameProperty displayNameProp = (DisplayNameProperty)response.getProperty(DavProperty.DISPLAYNAME);
                    ResourceTypeProperty resourceTypeProp = (ResourceTypeProperty)response.getProperty(DavProperty.RESOURCETYPE);
                    CreationDateProperty creationDateProp = (CreationDateProperty)response.getProperty(DavProperty.CREATIONDATE);
                    LastModifiedProperty lastModifiedProp = (LastModifiedProperty)response.getProperty(DavProperty.GETLASTMODIFIED);

                    WebDavProperty getContentLengthProp = response.getProperty("D:" + DavProperty.GETCONTENTLENGTH);
                    WebDavProperty versionNameProp = response.getProperty("D:version-name");

                    if (displayNameProp != null)
                    {
                        displayName = displayNameProp.getDisplayName();

                        int imageId = 0;
                        if (resourceTypeProp != null && resourceTypeProp.getResourceType() == ResourceTypeProperty.RESOURCE)
                        {
                            if (versionNameProp != null && versionNameProp.getStatus() == DavStatus.OK)
                            {
                                imageId = 2;
                            }
                            else
                            {
                                imageId = 1;
                            }

                        }

                        if (creationDateProp != null)
                        {
                            created = creationDateProp.getCreationDate();
                        }

                        if (lastModifiedProp != null)
                        {
                            modified = lastModifiedProp.getLastModified();
                        }

                        if (getContentLengthProp != null)
                        {
                            size = getContentLengthProp.getTextContent();
                        }

                        ListViewItem viewItem = new ListViewItem(new string[] {
                            displayName,
                            created,
                            modified,
                            size}, imageId);

                        listFiles.Items.Add(viewItem);

                    }

                }
            } catch (Exception exc) {
                MessageBox.Show("EXCEPTION " + exc.Message + " : " + exc.StackTrace);
            }

        }
        #endregion

        #region btnVersionList_Click(object sender, EventArgs e)
        private void btnVersionList_Click(object sender, EventArgs e)
        {
            if (listFiles.SelectedItems.Count == 0)
            {
                return;
            }

            ListViewItem curItem = listFiles.SelectedItems[0];
            activeAppContext.doVersionList();
        }
        #endregion

        private void button7_Click(object sender, EventArgs e)
        {

            if (listFiles.SelectedItems.Count == 0)
            {
                return;
            }

            ListViewItem curItem = listFiles.SelectedItems[0];
            MessageBox.Show(activeAppContext.getCurrentPath());
            activeAppContext.doPropertyList(activeAppContext.getCurrentPath());// + "/" + curItem.Text);
            
        }


        private void listFiles_Double_click (object sender, EventArgs e) 
        {
            int item_index = ((ListView)sender).FocusedItem.Index;
            activeAppContext.doGetFile(item_index);
        }

        #region listFiles_SelectedIndexChanged(object sender, System.EventArgs e)
        void listFiles_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            if (listFiles.SelectedItems.Count == 0)
            {
                return;
            }

            ListViewItem curItem = listFiles.SelectedItems[0];
            String oldpath = activeAppContext.getCurrentPath();
            oldpath = oldpath.Substring(0, oldpath.LastIndexOf("/"));
            if (oldpath != "")
                activeAppContext.setCurrentPath(oldpath+"/"+curItem.Text);
            else
                activeAppContext.setCurrentPath(activeAppContext.getCurrentPath() + "/" + curItem.Text);
        }
        #endregion

        private void button3_Click(object sender, EventArgs e)
        {
            int status = activeAppContext.getWindowContext().getFileList(remotePath);
            activeAppContext.refreshFileList(remotePath);
        }

    }
        
}
