/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS          All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using exo_jcr.webdav.csclient;
using exo_jcr.webdav.csclient.Request;
using exo_jcr.webdav.csclient.Commands;
using exo_jcr.webdav.csclient.Response;
using exo_jcr.webdav.csclient.DavProperties;

/**
 * Created by The eXo Platform SARL
 * Authors : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 *         : Max Shaposhnik <uy7c@yahoo.com>
 * @version $Id:
 */

namespace exo_jcr.msofficeplugin
{
    public partial class RemoteRepository : Form, AppWindowContext
    {
        private Hashtable multistatusCache = new Hashtable(); // <String, Multistatus>
        private Hashtable versionListTable = new Hashtable(); // <String, Multistatus>        

        private int status = 0;

        private WebDavAppContext appContext;

        public RemoteRepository(WebDavAppContext appContext)
        {
            this.appContext = appContext;
            
            InitializeComponent();

            activeTab1.setAppContext(this);
        }

        public DavContext getContext()
        {
            return appContext.getContext();
        }

        public WebDavAppContext getWebDavAppContext()
        {
            return appContext;
        }

        public Hashtable getMultistatusCache()
        {
            return multistatusCache;
        }

        public Hashtable getVersionListTable()
        {
            return versionListTable;
        }

        private void treeRepository_AfterSelect(object sender, TreeViewEventArgs e)
        {
            String serverPrefix = appContext.getContext().getContextHref();

            String path = e.Node.FullPath;
            if (path.StartsWith(serverPrefix))
            {
                path = path.Substring(serverPrefix.Length);

                if (path == "")
                {
                    path = "/";
                }

                activeTab1.setCurrentPath(path);
                if (multistatusCache[path] != null)
                {
                    activeTab1.refreshFileList();
                    return;
                }

                if (e.Node.ImageIndex == 2)
                {
                    return;
                }

                int status = getFileList(path);
                if (status == DavStatus.MULTISTATUS)
                {
                    fillTreeList(e.Node);
                    activeTab1.refreshFileList();
                }

            }

        }

        private void RemoteRepository_Load(object sender, EventArgs e)
        {
            TreeNode repositoryNode = treeRepository.Nodes.Add(appContext.getContext().getContextHref());
        }

        #region GET FILE LIST
        public int getFileList(String path)
        {
            try
            {
                PropFindCommand propFind = new PropFindCommand(appContext.getContext());
                propFind.setResourcePath(path);

                propFind.addRequiredProperty(DavProperty.DISPLAYNAME);
                propFind.addRequiredProperty(DavProperty.GETCONTENTTYPE);
                propFind.addRequiredProperty(DavProperty.RESOURCETYPE);
                propFind.addRequiredProperty(DavProperty.GETLASTMODIFIED);

                propFind.addRequiredProperty(DavProperty.GETCONTENTLENGTH);

                propFind.addRequiredProperty(DavProperty.CREATIONDATE);
                propFind.addRequiredProperty(DavProperty.HREF);
                propFind.addRequiredProperty(DavProperty.SUPPORTEDLOCK);
                propFind.addRequiredProperty("version-name");


                status = propFind.execute();

                byte[] data = propFind.getResponseBody();
                //String reply = System.Text.Encoding.UTF8.GetString(data);
                //MessageBox.Show("REPLY: " + reply);

                if (status == DavStatus.MULTISTATUS)
                {
                    if (multistatusCache[path] != null) {
                        multistatusCache.Remove(path);
                    }
                    multistatusCache.Add(path, propFind.getMultistatus());
                    //this.getDirectoryListing_success = true;
                }
                else
                {
                    MessageBox.Show("STATUS: " + status.ToString());
                }

                //this.getDirectoryListing_success = true;
                return status;
            }
            catch (Exception exc)
            {
                MessageBox.Show("Cannot load paramethers,\n please run Settings first.", "Error",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                //this.Close();
            }
            return -1;
        }
        #endregion

        private String getFullPath(TreeNode node)
        {
            String serverPrefix = appContext.getContext().getContextHref();
            String fullPath = node.FullPath.Substring(serverPrefix.Length);
            if (fullPath == "")
            {
                fullPath = "/";
            }
            return fullPath;
        }

        #region FILL TREE LIST
        public void fillTreeList(TreeNode node)
        {
            try
            {
                Multistatus multistatus = (Multistatus)multistatusCache[getFullPath(node)];
                ArrayList responses = multistatus.getResponses();
                for (int i = 0; i < responses.Count; i++)
                {
                    DavResponse response = (DavResponse)responses[i];

                    //MessageBox.Show("H1: [" + response.getHref().getHref() + "]\r\n" + 
                    //    "H2: [" + node.FullPath + "]");

                    if (response.getHref().getHref() == node.FullPath)
                    {
                        //MessageBox.Show("______");
                        continue;
                    }

                    DisplayNameProperty displayName = (DisplayNameProperty)response.getProperty(DavProperty.DISPLAYNAME);
                    ResourceTypeProperty resourceType = (ResourceTypeProperty)response.getProperty(DavProperty.RESOURCETYPE);
                    if (displayName != null)
                    {
                        TreeNode addedNode = node.Nodes.Add(displayName.getDisplayName());
                        if (resourceType != null && resourceType.getResourceType() == ResourceTypeProperty.RESOURCE)
                        {
                            addedNode.ImageIndex = 2;
                            addedNode.SelectedImageIndex = 2;
                        }
                        else
                        {
                            addedNode.ImageIndex = 1;
                            addedNode.SelectedImageIndex = 1;
                        }
                    }
                }

            }
            catch (Exception exc)
            {
                MessageBox.Show("EXCEPTION " + exc.Message + " : " + exc.StackTrace);
            }

        }
        #endregion

        
    }
}