/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS          All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;

using exo_jcr.webdav.csclient;
using exo_jcr.webdav.csclient.Commands;
using exo_jcr.webdav.csclient.Request;
using exo_jcr.webdav.csclient.Response;
using exo_jcr.webdav.csclient.DavProperties;

/**
 * Created by The eXo Platform SARL
 * Authors : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 *         : Max Shaposhnik <uy7c@yahoo.com>
 * @version $Id:
 */

namespace exo_jcr.msofficeplugin
{
    public partial class RepositoryView : Form
    {
        Word._Application app;
        //Connect connect;

        string serverPrefix = "http://localhost:8080/jcr-webdav/repository";

        private Hashtable multistatusCache = new Hashtable(); // <String, Multistatus>

        private int status = 0;

        private DavContext context = new DavContext("localhost", 8080, "/jcr-webdav/repository", "admin", "admin");
        
        public DavContext getContext()
        {            
            return context;
        }


        public RepositoryView(object app)
        {
            this.app = (Word._Application)app;
            //this.connect = connect;

            InitializeComponent();
        }

        private void RepositoryView_Load(object sender, EventArgs e)
        {
            TreeNode repositoryNode = treeRepository.Nodes.Add(serverPrefix);
        }

        #region TREE EVENTS

            private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
            {
                String path = e.Node.FullPath;
                if (path.StartsWith(serverPrefix))
                {
                    path = path.Substring(serverPrefix.Length);

                    if (path == "")
                    {
                        path = "/";
                    }

                    if (comboPath.Text != path) {
                        comboPath.Text = path;
                        comboPath.Items.Add(path);
                    }

                    if (multistatusCache[path] != null)
                    {
                        fillFileList(e.Node);
                        return;
                    }

                    if (e.Node.ImageIndex == 2) {
                        return;
                    }

                    int status = getFileList(path);
                    if (status == DavStatus.MULTISTATUS)
                    {
                        fillTreeList(e.Node);
                        fillFileList(e.Node);
                    }

                }

            }

        #endregion

        private String getFullPath(TreeNode node)
        {
            String fullPath = node.FullPath.Substring(serverPrefix.Length);
            if (fullPath == "")
            {
                fullPath = "/";
            }
            return fullPath;
        }

        #region GET FILE LIST
            public int getFileList(String path)
            {
                try
                {
                    PropFindCommand propFind = new PropFindCommand(context);
                    propFind.setResourcePath(path);

                    propFind.addRequiredProperty(DavProperty.DISPLAYNAME);
                    propFind.addRequiredProperty(DavProperty.GETCONTENTTYPE);
                    propFind.addRequiredProperty(DavProperty.RESOURCETYPE);
                    propFind.addRequiredProperty(DavProperty.GETLASTMODIFIED);
                    
                    propFind.addRequiredProperty(DavProperty.GETCONTENTLENGTH);
                    
                    propFind.addRequiredProperty(DavProperty.CREATIONDATE);
                    propFind.addRequiredProperty(DavProperty.HREF);
                    propFind.addRequiredProperty(DavProperty.SUPPORTEDLOCK);
                    propFind.addRequiredProperty("version-name");


                    status = propFind.execute();

                    byte[] data = propFind.getResponseBody();
                    //String reply = System.Text.Encoding.UTF8.GetString(data);
                    //MessageBox.Show("REPLY: " + reply);

                    if (status == DavStatus.MULTISTATUS)
                    {
                        multistatusCache.Add(path, propFind.getMultistatus());
                        //this.getDirectoryListing_success = true;
                    }
                    else
                    {
                        MessageBox.Show("STATUS: " + status.ToString());
                    }

                    //this.getDirectoryListing_success = true;
                    return status;
                }
                catch (Exception exc)
                {
                    MessageBox.Show("Cannot load paramethers,\n please run Settings first.", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                    //this.Close();
                }
                return -1;
            }
        #endregion

        #region FILL TREE LIST
        public void fillTreeList(TreeNode node)
        {
            try
            {
                Multistatus multistatus = (Multistatus)multistatusCache[getFullPath(node)];
                ArrayList responses = multistatus.getResponses();
                for (int i = 0; i < responses.Count; i++)
                {
                    DavResponse response = (DavResponse)responses[i];

                    if (response.getHref().getHref() == node.FullPath) {
                        //MessageBox.Show("______");
                        continue;
                    }

                    DisplayNameProperty displayName = (DisplayNameProperty)response.getProperty(DavProperty.DISPLAYNAME);
                    ResourceTypeProperty resourceType = (ResourceTypeProperty)response.getProperty(DavProperty.RESOURCETYPE);
                    if (displayName != null)
                    {
                        TreeNode addedNode = node.Nodes.Add(displayName.getDisplayName());
                        if (resourceType != null && resourceType.getResourceType() == ResourceTypeProperty.RESOURCE)
                        {
                            addedNode.ImageIndex = 2;
                            addedNode.SelectedImageIndex = 2;
                        }
                        else
                        {
                            addedNode.ImageIndex = 1;
                            addedNode.SelectedImageIndex = 1;
                        }
                    }
                }

            }
            catch (Exception exc)
            {
                MessageBox.Show("EXCEPTION " + exc.Message + " : " + exc.StackTrace);
            }

        }
        #endregion

        #region FILL FILE LIST
        private void fillFileList(TreeNode node)
        {
            listFiles.Items.Clear();
            try
            {
                Multistatus multistatus = (Multistatus)multistatusCache[getFullPath(node)];
                ArrayList responses = multistatus.getResponses();
                for (int i = 0; i < responses.Count; i++)
                {
                    DavResponse response = (DavResponse)responses[i];

                    String displayName = "";
                    String created = "";
                    String modified = "";
                    String size = "";

                    DisplayNameProperty displayNameProp = (DisplayNameProperty)response.getProperty(DavProperty.DISPLAYNAME);
                    ResourceTypeProperty resourceTypeProp = (ResourceTypeProperty)response.getProperty(DavProperty.RESOURCETYPE);
                    CreationDateProperty creationDateProp = (CreationDateProperty)response.getProperty(DavProperty.CREATIONDATE);
                    LastModifiedProperty lastModifiedProp = (LastModifiedProperty)response.getProperty(DavProperty.GETLASTMODIFIED);
                    
                    WebDavProperty getContentLengthProp = response.getProperty("D:" + DavProperty.GETCONTENTLENGTH);
                    WebDavProperty versionNameProp = response.getProperty("D:version-name");

                    //MessageBox.Show("CONTENT LENGTH: " + getContentLengthProp.getPropertyName());

                    if (displayNameProp != null)
                    {
                        displayName = displayNameProp.getDisplayName();

                        int imageId = 1;
                        if (resourceTypeProp != null && resourceTypeProp.getResourceType() == ResourceTypeProperty.RESOURCE) {
                            if (versionNameProp != null && versionNameProp.getStatus() == DavStatus.OK)
                            {
                                imageId = 3;
                            }
                            else
                            {
                                imageId = 2;
                            }
                            
                        }

                        if (creationDateProp != null) {
                            created = creationDateProp.getCreationDate();
                        }

                        if (lastModifiedProp != null) {
                            modified = lastModifiedProp.getLastModified();
                        }

                        if (getContentLengthProp != null) {
                            size = getContentLengthProp.getTextContent();
                        }
                        
                        ListViewItem viewItem = new ListViewItem(new string[] {
                            displayName,
                            created,
                            modified,
                            size}, imageId);

                        listFiles.Items.Add(viewItem);
                        
                    }
                }

            }
            catch (Exception exc)
            {
                MessageBox.Show("EXCEPTION " + exc.Message + " : " + exc.StackTrace);
            }
        }
        #endregion

        #region LIST FILES - CHANGE SELECTED
        private void listFiles_SelectedIndexChanged(object sender, EventArgs e)
        {
        }
        #endregion

        #region CONTEXT MENU LIST VERSIONS
        private void listVersionHistoryToolStripMenuItem_Click(object sender, EventArgs e)
        {

            int allCount = listFiles.SelectedItems.Count;
            if (allCount == 0) {
                return;
            }

            ListViewItem curItem = listFiles.SelectedItems[0];
            MessageBox.Show("ITEM: " + curItem.Text);

            String fullPath = comboPath.Text;
            MessageBox.Show("FULL PATH: [" + fullPath + "]");

        }
        #endregion

        private void largeIconsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            listFiles.View = View.LargeIcon;
        }

        private void smallIconsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            listFiles.View = View.SmallIcon;
        }

        private void listToolStripMenuItem_Click(object sender, EventArgs e)
        {
            listFiles.View = View.List;
        }

        private void detailsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            listFiles.View = View.Details;
        }

    }
}