/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS          All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Reflection;

using exo_jcr.webdav.csclient;
using exo_jcr.webdav.csclient.Commands;
using exo_jcr.webdav.csclient.Request;
using exo_jcr.webdav.csclient.Response;
using exo_jcr.webdav.csclient.DavProperties;
using exo_jcr.webdav.csclient.Search;

using System.Security.Permissions;
using Microsoft.Win32;

/**
 * Created by The eXo Platform SARL
 * Authors : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 *         : Max Shaposhnik <uy7c@yahoo.com>
 * @version $Id:
 */

[assembly: RegistryPermissionAttribute(SecurityAction.RequestMinimum,
    ViewAndModify = "HKEY_CURRENT_USER")]

namespace exo_jcr.msofficeplugin
{
    public partial class Search : Form
    {
        private Multistatus multistatus;

        Word._Application app;
        private Connect connect;
        private DavContext context;
        
        private static String SQL_TEXT   = "SQL Search";
        private static String XPATH_TEXT = "XPath Search";
        private static String BASIC_TEXT = "Basic Search";

        public DavContext getContext()
        {
            return context;
        }

        
        public Search(object app, Connect connect)
        {
            this.app = (Word._Application)app;
            this.connect = connect;
            InitializeComponent();
        }

        private void btn_cancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btn_search_Click(object sender, EventArgs e)
        {

             this.context = createContext();

            if (context == null) {
                MessageBox.Show("Cannot load paramethers,\n please run Settings first.", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();
            
            }



            SearchCommand searchcomm = new SearchCommand(context);
            searchcomm.setResourcePath("/production");
            String query_string = "";


            if (box_type.Text.Equals(""))
            {
                MessageBox.Show("Please, select Search Type", "Info");
                return;
            }
            
            // query_string = "//element(*, nt:file)";
            if (box_type.Text.Equals(SQL_TEXT))
            {
                query_string = "select * from nt:folder";
                SQLQuery query = new SQLQuery();
                query.setQuery(query_string);
                searchcomm.setQuery(query);
                
            }
            else if (box_type.Text.Equals(XPATH_TEXT))
            {
                query_string = "//element(*, nt:file)";
                XPathQuery query = new XPathQuery();
                query.setQuery(query_string);
                searchcomm.setQuery(query);
            }
          
            int status = searchcomm.execute();
            if (status == DavStatus.MULTISTATUS)
            {
                this.multistatus = searchcomm.getMultistatus();
                DrawFileList();
            }
            else if (status == DavStatus.NOT_FOUND)
            {
                MessageBox.Show("No results found.");
            }
            else {
                MessageBox.Show("Error in query.");
            }

        }

        private void Search_Load(object sender, EventArgs e)
        {
            this.box_type.Items.Add(SQL_TEXT);
            this.box_type.Items.Add(XPATH_TEXT);
        }


        private void DrawFileList()
        {
            
            this.file_list.Items.Clear();
            ArrayList resplist = multistatus.getResponses();
            for (int i = 0; i < resplist.Count; i++)
            {
                int image = 2;

                exo_jcr.webdav.csclient.Response.DavResponse response = (exo_jcr.webdav.csclient.Response.DavResponse)resplist[i];

                String href = response.getHref().getHref();
                if (href == null)
                {
                    href = "";
                }
                String displayName = "";
                WebDavProperty displayNameProp = response.getProperty(DavProperty.DISPLAYNAME);
                if (displayNameProp != null)
                {
                    displayName = ((DisplayNameProperty)displayNameProp).getDisplayName();
                }

                ResourceTypeProperty resourceTypeProp = (ResourceTypeProperty)response.getProperty(DavProperty.RESOURCETYPE);
                if (resourceTypeProp.getResourceType() == ResourceTypeProperty.COLLECTION)
                    image = 2;
                else
                    image = 3;

                String cdate = "";
                CreationDateProperty cdpropProp = (CreationDateProperty)response.getProperty(DavProperty.CREATIONDATE);
                if (cdpropProp != null)
                {
                    cdate = ((CreationDateProperty)cdpropProp).getCreationDate();
                }


                String lastmodified = "";
                LastModifiedProperty lastModifiedProp = (LastModifiedProperty)response.getProperty(DavProperty.GETLASTMODIFIED);
                if (lastModifiedProp != null)
                {
                    lastmodified = ((LastModifiedProperty)lastModifiedProp).getLastModified();
                }


                String clenght = "";// response.getProperty(DavProperty.GETCONTENTLENGTH).ToString();
                if (clenght == null)
                {
                    clenght = "";
                }

                System.Windows.Forms.ListViewItem Line = new System.Windows.Forms.ListViewItem(new string[] {
                    displayName,
                    href,
                    cdate,   
                    lastmodified,
                    clenght,
                    }, image);

                this.file_list.Items.Add(Line);
            }

        }


        private void file_list_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            int item_index = ((ListView)sender).FocusedItem.Index;

            ArrayList resplist = multistatus.getResponses();
            exo_jcr.webdav.csclient.Response.DavResponse response = (exo_jcr.webdav.csclient.Response.DavResponse)resplist[item_index];

            ResourceTypeProperty rtproperty = (ResourceTypeProperty)response.getProperty(DavProperty.RESOURCETYPE);
            if (rtproperty.getResourceType() == ResourceTypeProperty.RESOURCE)
            {

                String _path = response.getHref().getHref();
                OpenWordDocument(_path);

            }
            else
            {
                //String _href = response.getHref().getHref();
                //this.context = createContext(_href);

                //DrawComboPwd(basedir);
                //this.CurrentDir = basedir;
                //FillFileList_LaunchThread();
                //IncrementProgressBar_LaunchThread();

            }
        }


        private DavContext createContext()
        {
            RegistryKey soft_key = Registry.CurrentUser.OpenSubKey(RegKeys.SOFTWARE_KEY);
            try
            {

                RegistryKey exo_key = soft_key.OpenSubKey(RegKeys.EXO_KEY);
                RegistryKey client_key = exo_key.OpenSubKey(RegKeys.CLIENT_KEY);

                String _server = client_key.GetValue(RegKeys.S_ADDDR_KEY, "").ToString();
                int _port = System.Convert.ToInt32(client_key.GetValue(RegKeys.S_PORT_KEY, "").ToString());
                String _servlet = client_key.GetValue(RegKeys.S_SERVLET_KEY, "").ToString();
                String _username = client_key.GetValue(RegKeys.USER_KEY, "").ToString();
                String svalue = client_key.GetValue(RegKeys.PASS_KEY, "").ToString();
                byte[] bs_pass = System.Convert.FromBase64String(svalue);
                String _pass = Encoding.UTF8.GetString(bs_pass);
                /*
                    String servletPath = "";
                    String to_find = _server + ":" + _port;

                if (!url.Equals(""))
                {
                    servletPath = url.Substring(url.IndexOf(to_find) + to_find.Length);
                }

                if (servletPath != "")
                {
                    this.basedir = "http://" + _server + ":" + _port + servletPath;
                    return new exo_jcr.webdav.csclient.Request.DavContext(_server, _port, servletPath, _username, _pass);

                }
                else
                {
                    this.basedir = "http://" + _server + ":" + _port + _servlet;
                    return new exo_jcr.webdav.csclient.Request.DavContext(_server, _port, _servlet, _username, _pass);
                }
                */
                return new exo_jcr.webdav.csclient.Request.DavContext(_server, _port, _servlet, _username, _pass);

            }
            catch (Exception)
            {
                return null;

            }
        }

        private void btn_run_Click(object sender, EventArgs e)
        {
            this.context = createContext();
            if (context == null)
            {
                MessageBox.Show("Cannot load paramethers,\n please run Settings first.", "Error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();
            }

            SearchCommand searchcomm = new SearchCommand(context);
            searchcomm.setResourcePath("/production");
            String query_string = "";

            if (box_type.Text.Equals(""))
            {
                MessageBox.Show("Please, select Search Type.", "Info");
                return;
            }
            if (box_query.Text.Equals(""))
            {
                MessageBox.Show("Please, enter query.", "Info");
                return;
            }
           
            query_string = box_query.Text;

            if (box_type.Text.Equals(SQL_TEXT))
            {
                SQLQuery query = new SQLQuery();
                query.setQuery(query_string);
                searchcomm.setQuery(query);
            }
            else {
                XPathQuery query = new XPathQuery();
                query.setQuery(query_string);
                searchcomm.setQuery(query);
            }

            int status = searchcomm.execute();

            if (status == DavStatus.MULTISTATUS)
            {
                this.multistatus = searchcomm.getMultistatus();
                DrawFileList();
            }
            else if (status == DavStatus.NOT_FOUND)
            {
                MessageBox.Show("No results found.");
            }
            else
            {
                MessageBox.Show("Error in query.");
            }

        }

        private void OpenWordDocument(string resource)
        {
            Environment.SpecialFolder p = Environment.SpecialFolder.Personal;
            string s_p = Environment.GetFolderPath(p);
            string f_name = resource.Substring(resource.LastIndexOf("/") + 1);
            string FILE_NAME = s_p + "\\" + f_name;

            MessageBox.Show(FILE_NAME);
            this.context = createContext();
            GetCommand get = new GetCommand(context);
            get.setResourcePath(resource.Substring(resource.IndexOf(context.ServletPath) + context.ServletPath.Length));
            int status = get.execute();
            if (status == DavStatus.OK)
            {

                byte[] resp = get.getResponseBody();

                if (!File.Exists(FILE_NAME))
                {

                    FileStream fs = new FileStream(FILE_NAME, FileMode.CreateNew);
                    BinaryWriter w = new BinaryWriter(fs);
                    for (long i = 0; i < resp.Length; i++)
                    {
                        w.Write(resp[i]);
                    }
                    w.Close();
                    fs.Close();
                }

            }
            connect.Filename = FILE_NAME;
            this.Close();
        }
        
    }
}