/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS          All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using exo_jcr.webdav.csclient;
using exo_jcr.webdav.csclient.Commands;
using exo_jcr.webdav.csclient.Request;
using exo_jcr.webdav.csclient.Response;
using exo_jcr.webdav.csclient.DavProperties;
using exo_jcr.webdav.csclient.Search;

/**
 * Created by The eXo Platform SARL
 * Authors : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 *         : Max Shaposhnik <uy7c@yahoo.com>
 * @version $Id:
 */

namespace exo_jcr.msofficeplugin
{
    public partial class eXoSearchControl : UserControl
    {
        public eXoSearchControl()
        {
            InitializeComponent();
        }


        private DavContext context = new DavContext("localhost", 8080, "/jcr-webdav/repository", "admin", "admin");

        private Multistatus multistatus;

        public DavContext getContext()
        {
            return context;
        }


        private void btn_search_Click(object sender, EventArgs e)
        {
            try
            {

                SearchCommand searchcomm = new SearchCommand(context);
                searchcomm.setResourcePath("/production");
                String query_string = "";

                if (btn_sql.Checked == true)
                {
                    query_string = box_query.Text;
                    SQLQuery query = new SQLQuery();
                    query.setQuery(query_string);
                    searchcomm.setQuery(query);

                }
                else if (btn_xpath.Checked == true)
                {
                    query_string = box_query.Text;
                    XPathQuery query = new XPathQuery();
                    query.setQuery(query_string);
                    searchcomm.setQuery(query);
                }

                int status = searchcomm.execute();

                if (status == DavStatus.MULTISTATUS)
                {
                    this.multistatus = searchcomm.getMultistatus();
                    fillFileList();
                }
                else if (status == DavStatus.NOT_FOUND)
                {
                    MessageBox.Show("No results found.");
                }
                else
                {
                    MessageBox.Show("Error in query. Status: "+status);
                }

            }
            catch (Exception ext)
            {
                MessageBox.Show("EXC SEARCHCOMM:"+ext.Message+ext.StackTrace);
            }



        }


        private void fillFileList()
        {
            
            listfiles.Items.Clear();
            try
            {
                //Multistatus multistatus = (Multistatus)multistatusCache[getFullPath(node)];
                ArrayList responses = multistatus.getResponses();
                
                for (int i = 0; i < responses.Count; i++)
                {
                    DavResponse response = (DavResponse)responses[i];

                    String displayName = "";
                    String created = "";
                    String modified = "";
                    String size = "";

                    DisplayNameProperty displayNameProp = (DisplayNameProperty)response.getProperty(DavProperty.DISPLAYNAME);
                    ResourceTypeProperty resourceTypeProp = (ResourceTypeProperty)response.getProperty(DavProperty.RESOURCETYPE);
                    CreationDateProperty creationDateProp = (CreationDateProperty)response.getProperty(DavProperty.CREATIONDATE);
                    LastModifiedProperty lastModifiedProp = (LastModifiedProperty)response.getProperty(DavProperty.GETLASTMODIFIED);
                    
                    
                    WebDavProperty getContentLengthProp = response.getProperty(DavProperty.GETCONTENTLENGTH);
                    WebDavProperty versionNameProp = response.getProperty(DavProperty.VERSIONNAME);

                    //MessageBox.Show("CONTENT LENGTH: ");// + getContentLengthProp.getPropertyName());

                    if (displayNameProp != null)
                    {
                        displayName = displayNameProp.getDisplayName();

                        int imageId = 1;
                        if (resourceTypeProp != null && resourceTypeProp.getResourceType() == ResourceTypeProperty.RESOURCE)
                        {
                            if (versionNameProp != null && versionNameProp.getStatus() == DavStatus.OK)
                            {
                                imageId = 3;
                            }
                            else
                            {
                                imageId = 2;
                            }

                        }

                        if (creationDateProp != null)
                        {
                            created = creationDateProp.getCreationDate();
                        }

                        if (lastModifiedProp != null)
                        {
                            modified = lastModifiedProp.getLastModified();
                        }

                        if (getContentLengthProp != null)
                        {
                            size = getContentLengthProp.getTextContent();
                        }

                        ListViewItem viewItem = new ListViewItem(new string[] {
                            displayName,
                            created,
                            modified,
                            size}, imageId);

                        listfiles.Items.Add(viewItem);

                    }
                    //else MessageBox.Show("DN is NULL!!");
                }

            }
            catch (Exception exc)
            {
                MessageBox.Show("EXCEPTION " + exc.Message + " : " + exc.StackTrace);
            }
        }
      
    }
}
